@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: asmlink.mli 9081 2008-10-14 07:37:28Z maranget $ *)
+(* $Id: asmlink.mli 10507 2010-06-04 19:16:58Z maranget $ *)
 
 (* Link a set of .cmx/.o files and produce an executable or a plugin *)
 
@@ -22,7 +22,7 @@ val link_shared: formatter -> string list -> string -> unit
 
 val call_linker_shared: string list -> string -> unit
 
-val check_consistency: string -> Compilenv.unit_infos -> Digest.t -> unit
+val check_consistency: string -> Cmx_format.unit_infos -> Digest.t -> unit
 val extract_crc_interfaces: unit -> (string * Digest.t) list
 val extract_crc_implementations: unit -> (string * Digest.t) list
 
