@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: obj.c 9590 2010-01-25 14:52:00Z maranget $ */
+/* $Id: obj.c 10505 2010-06-04 19:16:46Z maranget $ */
 
 /* Operations on objects */
 
@@ -167,7 +167,7 @@ CAMLprim value caml_obj_truncate (value v, value newsize)
 
 CAMLprim value caml_obj_add_offset (value v, value offset)
 {
-  return v + Int32_val (offset);
+  return v + (unsigned long) Int32_val (offset);
 }
 
 /* The following functions are used in stdlib/lazy.ml.
