@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: stacks.c 7192 2005-10-27 09:14:16Z maranget $ */
+/* $Id: stacks.c 10505 2010-06-04 19:16:46Z maranget $ */
 
 /* To initialize and resize the stacks */
 
@@ -102,3 +102,14 @@ void caml_change_max_stack_size (uintnat new_max_size)
   }
   caml_max_stack_size = new_max_size;
 }
+
+CAMLexport uintnat (*caml_stack_usage_hook)(void) = NULL;
+
+uintnat caml_stack_usage(void)
+{
+  uintnat sz;
+  sz = caml_stack_high - caml_extern_sp;
+  if (caml_stack_usage_hook != NULL)
+    sz += (*caml_stack_usage_hook)();
+  return sz;
+}
