@@ -10,7 +10,7 @@
 ;(*                                                                     *)
 ;(***********************************************************************)
 
-;(* $Id: camldebug.el 7195 2005-10-27 09:52:37Z maranget $ *)
+;(* $Id: camldebug.el 10520 2010-06-04 19:19:48Z maranget $ *)
 
 ;;; Run camldebug under Emacs
 ;;; Derived from gdb.el.
@@ -573,7 +573,11 @@ the camldebug commands `cd DIR' and `directory'."
   (let ((output))
     (if (buffer-name (process-buffer proc))
         (let ((process-window))
-          (save-excursion
+          ;; it does not seem necessary to save excursion here,
+          ;; since set-buffer as a temporary effect.
+          ;; comint-output-filter explicitly avoids it. 
+          ;; in version 23, it prevents the marker to stay at end of buffer
+          ;; (save-excursion
             (set-buffer (process-buffer proc))
             ;; If we have been so requested, delete the debugger prompt.
             (if (marker-buffer camldebug-delete-prompt-marker)
@@ -590,7 +594,12 @@ the camldebug commands `cd DIR' and `directory'."
                                       (>= (point) (process-mark proc))
                                       (get-buffer-window (current-buffer))))
             ;; Insert the text, moving the process-marker.
-            (comint-output-filter proc output))
+            (comint-output-filter proc output)
+          ;; ) 
+          ;; this was the end of save-excursion. 
+          ;; if save-excursion is used (comint-next-prompt 1) would be needed
+          ;; to move the mark past then next prompt, but this is not as good
+          ;; as solution.
           (if process-window
               (save-selected-window
                 (select-window process-window)
