@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: main.ml 6857 2005-05-04 20:30:00Z habouzit $ *)
+(* $Id: main.ml 10502 2010-06-04 19:15:07Z maranget $ *)
 
 (* The lexer generator. Command-line parsing. *)
 
@@ -28,6 +28,11 @@ let print_version_string () =
   print_string Sys.ocaml_version ; print_newline();
   exit 0
 
+let print_version_num () =
+  print_endline Sys.ocaml_version;
+  exit 0;
+;;
+
 let specs =
   ["-ml", Arg.Set ml_automata,
     " Output code that does not use the Lexing module built-in automata interpreter";
@@ -36,7 +41,8 @@ let specs =
    "-q", Arg.Set Common.quiet_mode, " Do not display informational messages";
    "-v",  Arg.Unit print_version_string, " Print version and exit";
    "-version",  Arg.Unit print_version_string, " Print version and exit";
-  ] 
+   "-vnum",  Arg.Unit print_version_num, " Print version number and exit";
+  ]
 
 let _ =
   Arg.parse
@@ -44,11 +50,11 @@ let _ =
     (fun name -> source_name := Some name)
     usage
 
-  
+
 let main () =
 
   let source_name = match !source_name with
-  | None -> Arg.usage specs usage ; exit 2 
+  | None -> Arg.usage specs usage ; exit 2
   | Some name -> name in
   let dest_name = match !output_name with
   | Some name -> name
@@ -105,7 +111,7 @@ let main () =
     | Lexgen.Memory_overflow ->
         Printf.fprintf stderr
           "File \"%s\":\n Position memory overflow, too many bindings\n"
-          source_name        
+          source_name
     | Output.Table_overflow ->
         Printf.fprintf stderr
           "File \"%s\":\ntransition table overflow, automaton is too big\n"
@@ -116,4 +122,3 @@ let main () =
     exit 3
 
 let _ = (* Printexc.catch *) main (); exit 0
-
