@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: ns.ml 8298 2007-06-01 08:22:23Z maranget $ *)
+(* $Id: ns.ml 10494 2010-06-04 07:48:53Z maranget $ *)
 
 open Unix
 (*DEBUG*)open Printf
@@ -47,9 +47,14 @@ and do_register(key,v) =
 
 
 type t =
-   { lookup : 'a.string -> 'a ; register : 'a.(string * 'a) -> unit ; }
+   { lookup : 'a.string -> 'a ; register : 'a.(string * 'a) -> unit ;
+     site : Site.t ;
+   }
 
-let ns = { lookup = do_lookup ;  register = do_register ; }
+
+let ns = { lookup = do_lookup ;  register = do_register ;
+           site = Site.here
+         }
 
 let here = ns
   
@@ -64,6 +69,8 @@ let of_site site =
     Join_prim.call_service (Join_prim.space_id_of_chan site, name_key) () in
   (link : t)
 
+let to_site ns = ns.site
+
 let there sockaddr = of_site (Site.there sockaddr)
 
 let of_sockaddr = there
