@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: lseek.c 6336 2004-05-27 09:18:38Z maranget $ */
+/* $Id: lseek.c 10509 2010-06-04 19:17:18Z maranget $ */
 
 #include <errno.h>
 #include <sys/types.h>
@@ -39,8 +39,10 @@ static int seek_command_table[] = {
 CAMLprim value unix_lseek(value fd, value ofs, value cmd)
 {
   file_offset ret;
+  caml_enter_blocking_section();
   ret = lseek(Int_val(fd), Long_val(ofs),
                        seek_command_table[Int_val(cmd)]);
+  caml_leave_blocking_section();
   if (ret == -1) uerror("lseek", Nothing);
   if (ret > Max_long) unix_error(EOVERFLOW, "lseek", Nothing);
   return Val_long(ret);
@@ -49,9 +51,13 @@ CAMLprim value unix_lseek(value fd, value ofs, value cmd)
 CAMLprim value unix_lseek_64(value fd, value ofs, value cmd)
 {
   file_offset ret;
-  ret = lseek(Int_val(fd), File_offset_val(ofs),
-                       seek_command_table[Int_val(cmd)]);
+  /* [ofs] is an Int64, which is stored as a custom block; we must therefore
+     extract its contents before dropping the runtime lock, or it might be
+     moved. */
+  file_offset ofs_c = File_offset_val(ofs);
+  caml_enter_blocking_section();
+  ret = lseek(Int_val(fd), ofs_c, seek_command_table[Int_val(cmd)]);
+  caml_leave_blocking_section();
   if (ret == -1) uerror("lseek", Nothing);
   return Val_file_offset(ret);
 }
-
