@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: array.ml 7198 2005-10-27 11:22:54Z maranget $ *)
+(* $Id: array.ml 10510 2010-06-04 19:17:45Z maranget $ *)
 
 (* Array operations *)
 
@@ -78,10 +78,11 @@ let concat_aux init al =
 ;;
 
 let concat al =
-  let rec find_init = function
-      [] -> [||]
+  let rec find_init aa =
+    match aa with
+    | [] -> [||]
     | a :: rem ->
-        if length a > 0 then concat_aux (unsafe_get a 0) al else find_init rem
+        if length a > 0 then concat_aux (unsafe_get a 0) aa else find_init rem
   in find_init al
 
 let sub a ofs len =
