@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: printf.mli 9590 2010-01-25 14:52:00Z maranget $ *)
+(* $Id: printf.mli 10510 2010-06-04 19:17:45Z maranget $ *)
 
 (** Formatted output functions. *)
 
@@ -111,7 +111,9 @@ val eprintf : ('a, out_channel, unit) format -> 'a
 
 val ifprintf : 'a -> ('b, 'a, unit) format -> 'b
 (** Same as {!Printf.fprintf}, but does not print anything.
-    Useful to ignore some material when conditionally printing. *)
+    Useful to ignore some material when conditionally printing.
+    @since 3.10.0
+*)
 
 val sprintf : ('a, unit, string) format -> 'a
 (** Same as {!Printf.fprintf}, but instead of printing on an output channel,
@@ -127,16 +129,22 @@ val bprintf : Buffer.t -> ('a, Buffer.t, unit) format -> 'a
 val kfprintf : (out_channel -> 'a) -> out_channel ->
               ('b, out_channel, unit, 'a) format4 -> 'b;;
 (** Same as [fprintf], but instead of returning immediately,
-   passes the out channel to its first argument at the end of printing. *)
+   passes the out channel to its first argument at the end of printing.
+   @since 3.09.0
+*)
 
 val ksprintf : (string -> 'a) -> ('b, unit, string, 'a) format4 -> 'b;;
 (** Same as [sprintf] above, but instead of returning the string,
-   passes it to the first argument. *)
+   passes it to the first argument.
+   @since 3.09.0
+*)
 
 val kbprintf : (Buffer.t -> 'a) -> Buffer.t ->
               ('b, Buffer.t, unit, 'a) format4 -> 'b;;
 (** Same as [bprintf], but instead of returning immediately,
-   passes the buffer to its first argument at the end of printing. *)
+   passes the buffer to its first argument at the end of printing.
+   @since 3.10.0
+*)
 
 val kprintf : (string -> 'a) -> ('b, unit, string, 'a) format4 -> 'b;;
 (** A deprecated synonym for [ksprintf]. *)
