@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: checkstack.c 8113 2007-03-23 09:01:11Z maranget $ */
+/* $Id: checkstack.c 10514 2010-06-04 19:18:21Z maranget $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,7 +35,8 @@ int main(int argc, char ** argv)
              "Under sh, bash, zsh:  ulimit -s %lu\n"
              "Under csh, tcsh:      limit stacksize %lu\n\n",
              (unsigned long) (limit.rlim_cur / 1024),
-             MINSTACKBYTES / 1024, MINSTACKBYTES / 1024);
+             (unsigned long) (MINSTACKBYTES / 1024),
+             (unsigned long) (MINSTACKBYTES / 1024));
     exit (3);
   }
   exit (0);
