@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: env.ml 9301 2009-06-19 07:11:12Z maranget $ *)
+(* $Id: env.ml 10508 2010-06-04 19:17:06Z maranget $ *)
 
 (* Environment handling *)
 
@@ -685,7 +685,7 @@ let components_of_functor_appl f p1 p2 =
     Hashtbl.find f.fcomp_cache p2
   with Not_found ->
     let p = Papply(p1, p2) in
-    let mty = 
+    let mty =
       Subst.modtype (Subst.add_module f.fcomp_param p2 Subst.identity)
                     f.fcomp_res in
     let comps = components_of_module f.fcomp_env f.fcomp_subst p mty in
@@ -732,6 +732,13 @@ and add_continuation id desc env  =
   {env with continuations = new_conts}
 
 and remove_continuations t =  {t with continuations = Ident.empty}
+
+let do_purge (path,d as c) = match d.val_kind with
+  | Val_channel _|Val_alone _ -> path,{ d with val_kind = Val_reg; }
+  | _ -> c
+
+let remove_channel_info t =
+  { t with values = Ident.map do_purge t.values ; }
 (*< JOCAML *)
 
 (* Insertion of bindings by name *)
@@ -797,7 +804,7 @@ let open_signature root sg env =
                          (Subst.cltype_declaration sub decl) env)
       env sg pl in
   { newenv with summary = Env_open(env.summary, root) }
-  
+
 (* Open a signature from a file *)
 
 let open_pers_signature name env =
