@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: typedtree.mli 9081 2008-10-14 07:37:28Z maranget $ *)
+(* $Id: typedtree.mli 10508 2010-06-04 19:17:06Z maranget $ *)
 
 (* Abstract syntax tree after typing *)
 
@@ -78,6 +78,7 @@ and expression_desc =
   | Texp_assertfalse
   | Texp_lazy of expression
   | Texp_object of class_structure * class_signature * string list
+  | Texp_pack of module_expr
 (*> JOCAML *)
   | Texp_asyncsend of expression * expression
   | Texp_spawn of expression (* insert processes in expressions *)
@@ -87,6 +88,7 @@ and expression_desc =
   | Texp_def of joinautomaton list * expression
   | Texp_loc of joinlocation list * expression
 
+
 and joinlocation =
     {jloc_desc : joinident * joinautomaton list * expression ;
     jloc_loc : Location.t}
@@ -187,6 +189,7 @@ and module_expr_desc =
   | Tmod_functor of Ident.t * module_type * module_expr
   | Tmod_apply of module_expr * module_expr * module_coercion
   | Tmod_constraint of module_expr * module_type * module_coercion
+  | Tmod_unpack of expression * module_type
 
 and structure = structure_item list
 
@@ -233,4 +236,3 @@ val rev_loc_bound_idents: joinlocation list -> Ident.t list
 
 (* Alpha conversion of patterns *)
 val alpha_pat : (Ident.t * Ident.t) list -> pattern -> pattern
-
