@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: typemod.ml 9158 2008-12-12 16:38:05Z maranget $ *)
+(* $Id: typemod.ml 10611 2010-06-29 13:52:11Z maranget $ *)
 
 (* Type-checking of the module language *)
 
@@ -24,9 +24,7 @@ open Typedtree
 open Format
 
 type error =
-    Unbound_module of Longident.t
-  | Unbound_modtype of Longident.t
-  | Cannot_apply of module_type
+    Cannot_apply of module_type
   | Not_included of Includemod.error list
   | Cannot_eliminate_dependency of module_type
   | Signature_expected
@@ -39,6 +37,8 @@ type error =
   | Non_generalizable_module of module_type
   | Implementation_is_required of string
   | Interface_not_compiled of string
+  | Not_allowed_in_functor_body
+  | With_need_typeconstr
 
 exception Error of Location.t * error
 
@@ -54,19 +54,23 @@ let extract_sig_open env loc mty =
     Tmty_signature sg -> sg
   | _ -> raise(Error(loc, Structure_expected mty))
 
-(* Lookup the type of a module path *)
+(* Compute the environment after opening a module *)
 
-let type_module_path env loc lid =
-  try
-    Env.lookup_module lid env
-  with Not_found ->
-    raise(Error(loc, Unbound_module lid))
+let type_open env loc lid =
+  let (path, mty) = Typetexp.find_module env loc lid in
+  let sg = extract_sig_open env loc mty in
+  Env.open_signature path sg env
 
 (* Record a module type *)
 let rm node =
   Stypes.record (Stypes.Ti_mod node);
   node
 
+(* Forward declaration, to be filled in by type_module_type_of *)
+let type_module_type_of_fwd 
+  : (Env.t -> Parsetree.module_expr -> module_type) ref
+  = ref (fun env m -> assert false)
+
 (* Merge one "with" constraint in a signature *)
 
 let rec add_rec_types env = function
@@ -81,7 +85,14 @@ let check_type_decl env id row_id newdecl decl rs rem =
   let env = if rs = Trec_not then env else add_rec_types env rem in
   Includemod.type_declarations env id newdecl decl
 
+let rec make_params n = function
+    [] -> []
+  | _ :: l -> ("a" ^ string_of_int n) :: make_params (n+1) l
+
+let wrap_param s = {ptyp_desc=Ptyp_var s; ptyp_loc=Location.none}
+
 let merge_constraint initial_env loc sg lid constr =
+  let real_id = ref None in
   let rec merge env sg namelist row_id =
     match (sg, namelist, constr) with
       ([], _, _) ->
@@ -113,15 +124,30 @@ let merge_constraint initial_env loc sg lid constr =
           Typedecl.transl_with_constraint initial_env id None sdecl in
         check_type_decl env id row_id newdecl decl rs rem;
         Tsig_type(id, newdecl, rs) :: rem
-    | (Tsig_type(id, decl, rs) :: rem, [s], Pwith_type sdecl)
+    | (Tsig_type(id, decl, rs) :: rem, [s], (Pwith_type _ | Pwith_typesubst _))
       when Ident.name id = s ^ "#row" ->
         merge env rem namelist (Some id)
+    | (Tsig_type(id, decl, rs) :: rem, [s], Pwith_typesubst sdecl)
+      when Ident.name id = s ->
+        (* Check as for a normal with constraint, but discard definition *)
+        let newdecl =
+          Typedecl.transl_with_constraint initial_env id None sdecl in
+        check_type_decl env id row_id newdecl decl rs rem;
+        real_id := Some id;
+        rem
     | (Tsig_module(id, mty, rs) :: rem, [s], Pwith_module lid)
       when Ident.name id = s ->
-        let (path, mty') = type_module_path initial_env loc lid in
+        let (path, mty') = Typetexp.find_module initial_env loc lid in
         let newmty = Mtype.strengthen env mty' path in
         ignore(Includemod.modtypes env newmty mty);
         Tsig_module(id, newmty, rs) :: rem
+    | (Tsig_module(id, mty, rs) :: rem, [s], Pwith_modsubst lid)
+      when Ident.name id = s ->
+        let (path, mty') = Typetexp.find_module initial_env loc lid in
+        let newmty = Mtype.strengthen env mty' path in
+        ignore(Includemod.modtypes env newmty mty);
+        real_id := Some id;
+        rem
     | (Tsig_module(id, mty, rs) :: rem, s :: namelist, _)
       when Ident.name id = s ->
         let newsg = merge env (extract_sig env loc mty) namelist None in
@@ -129,7 +155,37 @@ let merge_constraint initial_env loc sg lid constr =
     | (item :: rem, _, _) ->
         item :: merge (Env.add_item item env) rem namelist row_id in
   try
-    merge initial_env sg (Longident.flatten lid) None
+    let names = Longident.flatten lid in
+    let sg = merge initial_env sg names None in
+    match names, constr with
+      [s], Pwith_typesubst sdecl ->
+        let id =
+          match !real_id with None -> assert false | Some id -> id in
+        let lid =
+          try match sdecl.ptype_manifest with
+          | Some {ptyp_desc = Ptyp_constr (lid, stl)} ->
+              let params =
+                List.map
+                  (function {ptyp_desc=Ptyp_var s} -> s | _ -> raise Exit)
+                  stl in
+              if params <> sdecl.ptype_params then raise Exit;
+              lid
+          | _ -> raise Exit
+          with Exit -> raise (Error (sdecl.ptype_loc, With_need_typeconstr))
+        in
+        let (path, _) =
+          try Env.lookup_type lid initial_env with Not_found -> assert false
+        in
+        let sub = Subst.add_type id path Subst.identity in
+        Subst.signature sub sg
+    | [s], Pwith_modsubst lid ->
+        let id =
+          match !real_id with None -> assert false | Some id -> id in
+        let (path, _) = Typetexp.find_module initial_env loc lid in
+        let sub = Subst.add_module id path Subst.identity in
+        Subst.signature sub sg
+    | _ ->
+        sg
   with Includemod.Error explanation ->
     raise(Error(loc, With_mismatch(lid, explanation)))
 
@@ -156,12 +212,8 @@ let rec map_rec' fn decls rem =
 let rec approx_modtype env smty =
   match smty.pmty_desc with
     Pmty_ident lid ->
-      begin try
-        let (path, info) = Env.lookup_modtype lid env in
-        Tmty_ident path
-      with Not_found ->
-        raise(Error(smty.pmty_loc, Unbound_modtype lid))
-      end
+      let (path, info) = Typetexp.find_modtype env smty.pmty_loc lid in
+      Tmty_ident path
   | Pmty_signature ssg ->
       Tmty_signature(approx_sig env ssg)
   | Pmty_functor(param, sarg, sres) ->
@@ -171,6 +223,8 @@ let rec approx_modtype env smty =
       Tmty_functor(id, arg, res)
   | Pmty_with(sbody, constraints) ->
       approx_modtype env sbody
+  | Pmty_typeof smod ->
+      !type_module_type_of_fwd env smod
 
 and approx_sig env ssg =
   match ssg with
@@ -201,10 +255,7 @@ and approx_sig env ssg =
           let (id, newenv) = Env.enter_modtype name info env in
           Tsig_modtype(id, info) :: approx_sig newenv srem
       | Psig_open lid ->
-          let (path, mty) = type_module_path env item.psig_loc lid in
-          let sg = extract_sig_open env item.psig_loc mty in
-          let newenv = Env.open_signature path sg env in
-          approx_sig newenv srem
+          approx_sig (type_open env item.psig_loc lid) srem
       | Psig_include smty ->
           let mty = approx_modtype env smty in
           let sg = Subst.signature Subst.identity
@@ -263,17 +314,26 @@ let check_sig_item type_names module_names modtype_names loc = function
       check "module type" loc modtype_names (Ident.name id)
   | _ -> ()
 
+let rec remove_values ids = function
+    [] -> []
+  | Tsig_value (id, _) :: rem when List.exists (Ident.equal id) ids -> rem
+  | f :: rem -> f :: remove_values ids rem
+
+let rec get_values = function
+    [] -> []
+  | Tsig_value (id, _) :: rem -> id :: get_values rem
+  | f :: rem -> get_values rem
+
 (* Check and translate a module type expression *)
 
+let transl_modtype_longident loc env lid =
+  let (path, info) = Typetexp.find_modtype env loc lid in
+  path
+
 let rec transl_modtype env smty =
   match smty.pmty_desc with
     Pmty_ident lid ->
-      begin try
-        let (path, info) = Env.lookup_modtype lid env in
-        Tmty_ident path
-      with Not_found ->
-        raise(Error(smty.pmty_loc, Unbound_modtype lid))
-      end
+      Tmty_ident (transl_modtype_longident smty.pmty_loc env lid)
   | Pmty_signature ssg ->
       Tmty_signature(transl_signature env ssg)
   | Pmty_functor(param, sarg, sres) ->
@@ -290,6 +350,8 @@ let rec transl_modtype env smty =
             merge_constraint env smty.pmty_loc sg lid sdecl)
           init_sg constraints in
       Mtype.freshen (Tmty_signature final_sg)
+  | Pmty_typeof smod ->
+      !type_module_type_of_fwd env smod
 
 and transl_signature env sg =
   let type_names = ref StringSet.empty
@@ -305,7 +367,8 @@ and transl_signature env sg =
             let desc = Typedecl.transl_value_decl env sdesc in
             let (id, newenv) = Env.enter_value name desc env in
             let rem = transl_sig newenv srem in
-            Tsig_value(id, desc) :: rem
+            if List.exists (Ident.equal id) (get_values rem) then rem
+            else Tsig_value(id, desc) :: rem
         | Psig_type sdecls ->
             List.iter
               (fun (name, decl) -> check "type" item.psig_loc type_names name)
@@ -340,10 +403,7 @@ and transl_signature env sg =
             let rem = transl_sig newenv srem in
             Tsig_modtype(id, info) :: rem
         | Psig_open lid ->
-            let (path, mty) = type_module_path env item.psig_loc lid in
-            let sg = extract_sig_open env item.psig_loc mty in
-            let newenv = Env.open_signature path sg env in
-            transl_sig newenv srem
+            transl_sig (type_open env item.psig_loc lid) srem
         | Psig_include smty ->
             let mty = transl_modtype env smty in
             let sg = Subst.signature Subst.identity
@@ -354,7 +414,7 @@ and transl_signature env sg =
               sg;
             let newenv = Env.add_signature sg env in
             let rem = transl_sig newenv srem in
-            sg @ rem
+            remove_values (get_values rem) sg @ rem
         | Psig_class cl ->
             List.iter
               (fun {pci_name = name} ->
@@ -429,7 +489,7 @@ exception Not_a_path
 let rec path_of_module mexp =
   match mexp.mod_desc with
     Tmod_ident p -> p
-  | Tmod_apply(funct, arg, coercion) ->
+  | Tmod_apply(funct, arg, coercion) when !Clflags.applicative_functors ->
       Papply(path_of_module funct, path_of_module arg)
   | _ -> raise Not_a_path
 
@@ -514,7 +574,7 @@ let check_recmodule_inclusion env bindings =
         module rec X : DECL = MOD  where MOD has inferred type ACTUAL
      The "natural" typing condition
         E, X: ACTUAL |- ACTUAL <: DECL
-     leads to circularities through manifest types.  
+     leads to circularities through manifest types.
      Instead, we "unroll away" the potential circularities a finite number
      of times.  The (weaker) condition we implement is:
         E, X: DECL,
@@ -584,16 +644,17 @@ let check_recmodule_inclusion env bindings =
 
 (* Type a module value expression *)
 
-let rec type_module anchor env smod =
+let rec type_module sttn funct_body anchor env smod =
   match smod.pmod_desc with
     Pmod_ident lid ->
-      let (path, mty) = type_module_path env smod.pmod_loc lid in
+      let (path, mty) = Typetexp.find_module env smod.pmod_loc lid in
       rm { mod_desc = Tmod_ident path;
-           mod_type = Mtype.strengthen env mty path;
+           mod_type = if sttn then Mtype.strengthen env mty path else mty;
            mod_env = env;
            mod_loc = smod.pmod_loc }
   | Pmod_structure sstr ->
-      let (str, sg, finalenv) = type_structure anchor env sstr smod.pmod_loc in
+      let (str, sg, finalenv) =
+        type_structure funct_body anchor env sstr smod.pmod_loc in
       rm { mod_desc = Tmod_structure str;
            mod_type = Tmty_signature sg;
            mod_env = env;
@@ -601,14 +662,16 @@ let rec type_module anchor env smod =
   | Pmod_functor(name, smty, sbody) ->
       let mty = transl_modtype env smty in
       let (id, newenv) = Env.enter_module name mty env in
-      let body = type_module None newenv sbody in
+      let body = type_module sttn true None newenv sbody in
       rm { mod_desc = Tmod_functor(id, mty, body);
            mod_type = Tmty_functor(id, mty, body.mod_type);
            mod_env = env;
            mod_loc = smod.pmod_loc }
   | Pmod_apply(sfunct, sarg) ->
-      let funct = type_module None env sfunct in
-      let arg = type_module None env sarg in
+      let arg = type_module true funct_body None env sarg in
+      let path = try Some (path_of_module arg) with Not_a_path -> None in
+      let funct =
+        type_module (sttn && path <> None) funct_body None env sfunct in
       begin match Mtype.scrape env funct.mod_type with
         Tmty_functor(param, mty_param, mty_res) as mty_functor ->
           let coercion =
@@ -617,17 +680,18 @@ let rec type_module anchor env smod =
             with Includemod.Error msg ->
               raise(Error(sarg.pmod_loc, Not_included msg)) in
           let mty_appl =
-            try
-              let path = path_of_module arg in
-              Subst.modtype (Subst.add_module param path Subst.identity)
-                            mty_res
-            with Not_a_path ->
-              try
-                Mtype.nondep_supertype
-                  (Env.add_module param arg.mod_type env) param mty_res
-              with Not_found ->
-                raise(Error(smod.pmod_loc,
-                            Cannot_eliminate_dependency mty_functor)) in
+            match path with
+              Some path ->
+                Subst.modtype (Subst.add_module param path Subst.identity)
+                              mty_res
+            | None ->
+                try
+                  Mtype.nondep_supertype
+                    (Env.add_module param arg.mod_type env) param mty_res
+                with Not_found ->
+                  raise(Error(smod.pmod_loc,
+                              Cannot_eliminate_dependency mty_functor))
+          in
           rm { mod_desc = Tmod_apply(funct, arg, coercion);
                mod_type = mty_appl;
                mod_env = env;
@@ -636,7 +700,7 @@ let rec type_module anchor env smod =
           raise(Error(sfunct.pmod_loc, Cannot_apply funct.mod_type))
       end
   | Pmod_constraint(sarg, smty) ->
-      let arg = type_module anchor env sarg in
+      let arg = type_module true funct_body anchor env sarg in
       let mty = transl_modtype env smty in
       let coercion =
         try
@@ -648,7 +712,19 @@ let rec type_module anchor env smod =
            mod_env = env;
            mod_loc = smod.pmod_loc }
 
-and type_structure anchor env sstr scope =
+  | Pmod_unpack (sexp, (p, l)) ->
+      if funct_body then
+        raise (Error (smod.pmod_loc, Not_allowed_in_functor_body));
+      let l, mty = Typetexp.create_package_mty smod.pmod_loc env (p, l) in
+      let mty = transl_modtype env mty in
+      let exp = Typecore.type_expect env sexp
+          (Typecore.create_package_type smod.pmod_loc env (p, l)) in
+      rm { mod_desc = Tmod_unpack(exp, mty);
+           mod_type = mty;
+           mod_env = env;
+           mod_loc = smod.pmod_loc }
+
+and type_structure funct_body anchor env sstr scope =
   let type_names = ref StringSet.empty
   and module_names = ref StringSet.empty
   and modtype_names = ref StringSet.empty in
@@ -737,7 +813,9 @@ and type_structure anchor env sstr scope =
          final_env)
     | {pstr_desc = Pstr_module(name, smodl); pstr_loc = loc} :: srem ->
         check "module" loc module_names name;
-        let modl = type_module  (anchor_submodule name anchor) env smodl in
+        let modl =
+          type_module true funct_body (anchor_submodule name anchor) env
+            smodl in
         let mty = enrich_module_type anchor name modl.mod_type env in
         let (id, newenv) = Env.enter_module name mty env in
         let (str_rem, sig_rem, final_env) = type_struct newenv srem in
@@ -755,9 +833,11 @@ and type_structure anchor env sstr scope =
           List.map2
             (fun (id, mty) (name, smty, smodl) ->
               let modl =
-                type_module (anchor_recmodule id anchor) newenv smodl in
+                type_module true funct_body (anchor_recmodule id anchor) newenv
+                  smodl in
               let mty' =
-                enrich_module_type anchor (Ident.name id) modl.mod_type newenv in
+                enrich_module_type anchor (Ident.name id) modl.mod_type newenv
+              in
               (id, mty, modl, mty'))
            decls sbind in
         let bindings2 =
@@ -776,9 +856,7 @@ and type_structure anchor env sstr scope =
          Tsig_modtype(id, Tmodtype_manifest mty) :: sig_rem,
          final_env)
     | {pstr_desc = Pstr_open lid; pstr_loc = loc} :: srem ->
-        let (path, mty) = type_module_path env loc lid in
-        let sg = extract_sig_open env loc mty in
-        type_struct (Env.open_signature path sg env) srem
+        type_struct (type_open env loc lid) srem
     | {pstr_desc = Pstr_class cl; pstr_loc = loc} :: srem ->
          List.iter
            (fun {pci_name = name} -> check "type" loc type_names name)
@@ -827,7 +905,7 @@ and type_structure anchor env sstr scope =
               classes [sig_rem]),
          final_env)
     | {pstr_desc = Pstr_include smodl; pstr_loc = loc} :: srem ->
-        let modl = type_module None env smodl in
+        let modl = type_module true funct_body None env smodl in
         (* Rename all identifiers bound by this signature to avoid clashes *)
         let sg = Subst.signature Subst.identity
                    (extract_sig_open env smodl.pmod_loc modl.mod_type) in
@@ -843,12 +921,8 @@ and type_structure anchor env sstr scope =
   then List.iter (function {pstr_loc = l} -> Stypes.record_phrase l) sstr;
   type_struct env sstr
 
-let type_module = type_module None
-let type_structure = type_structure None
-
-(* Fill in the forward declaration *)
-let _ =
-  Typecore.type_module := type_module
+let type_module = type_module true false None
+let type_structure = type_structure false None
 
 (* Normalize types in a signature *)
 
@@ -896,6 +970,29 @@ and simplify_signature sg =
   in
     simplif StringSet.empty StringSet.empty [] (List.rev sg)
 
+(* Extract the module type of a module expression *)
+
+let type_module_type_of env smod =
+  let mty =
+    match smod.pmod_desc with
+    | Pmod_ident lid -> (* turn off strengthening in this case *)
+        let (path, mty) = Typetexp.find_module env smod.pmod_loc lid in mty
+    | _ -> (type_module env smod).mod_type in
+  (* PR#5037: clean up inferred signature to remove duplicate specs *)
+  let mty = simplify_modtype mty in
+  (* PR#5036: must not contain non-generalized type variables *)
+  if not (closed_modtype mty) then
+    raise(Error(smod.pmod_loc, Non_generalizable_module mty));
+  mty
+
+(* Fill in the forward declarations *)
+let () =
+  Typecore.type_module := type_module;
+  Typetexp.transl_modtype_longident := transl_modtype_longident;
+  Typetexp.transl_modtype := transl_modtype;
+  Typecore.type_open := type_open;
+  type_module_type_of_fwd := type_module_type_of
+
 (* Typecheck an implementation file *)
 
 let type_implementation sourcefile outputprefix modulename initial_env ast =
@@ -984,9 +1081,7 @@ let package_units objfiles cmifile modulename =
 open Printtyp
 
 let report_error ppf = function
-  | Unbound_module lid -> fprintf ppf "Unbound module %a" longident lid
-  | Unbound_modtype lid -> fprintf ppf "Unbound module type %a" longident lid
-  | Cannot_apply mty ->
+    Cannot_apply mty ->
       fprintf ppf
         "@[This module is not a functor; it has type@ %a@]" modtype mty
   | Not_included errs ->
@@ -1037,3 +1132,9 @@ let report_error ppf = function
   | Interface_not_compiled intf_name ->
       fprintf ppf
         "@[Could not find the .cmi file for interface@ %s.@]" intf_name
+  | Not_allowed_in_functor_body ->
+      fprintf ppf
+        "This kind of expression is not allowed within the body of a functor."
+  | With_need_typeconstr ->
+      fprintf ppf
+        "Only type constructors with identical parameters can be substituted."
